﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectMachine
{
    /// <summary>
    /// Параметризованный набор объектов
    /// </summary>
    /// <typeparam name="T"></typeparam>
    internal class SetTankGeneric<T>
        where T : class
    {
        /// <summary>
        /// Список объектов, которые храним
        /// </summary>
        private readonly List<T> _places;
        /// <summary>
        /// Количество объектов в списке
        /// </summary>
        public int Count => _places.Count;
        private readonly int _maxCount; 
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="count"></param>
        public SetTankGeneric(int count)
        {
            _maxCount = count;
            _places = new List<T>();
        }
        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="tank">Добавляемый танк</param>
        /// <returns></returns>
        public int Insert(T tank)
        {
            if (Count < _maxCount)
                return Insert(tank, 0);
            return -1;
        }
        /// <summary>
        /// Добавление объекта в набор на конкретную позицию
        /// </summary>
        /// <param name="tank">Добавляемый танк</param>
        /// <param name="position">Позиция</param>
        /// <returns></returns>
        public int Insert(T tank, int position)
        {
            if (position < 0 || position >= _maxCount)
                return -1;
            _places.Insert(position, tank);
            return position;
        }
        /// <summary>
        /// Удаление объекта из набора с конкретной позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T Remove(int position)
        {
            if (position < 0 || position >= Count)
                return null;
            var result = _places[position];
            _places.RemoveAt(position);
            return result;
        }
        /// <summary>
        /// Получение объекта из набора по позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T this[int position]
        {
            get
            {
                if (position < 0 || position >= Count)
                    return null;
                return _places[position];
            }
            set
            {
                if (position < 0 || position >= _maxCount)
                    return;
                _places.Add(value);
            }
        }
        /// <summary>
        /// Проход по набору до первого пустого
        /// </summary>
        /// <returns></returns>
        public IEnumerable<T> GetTanks()
        {
            foreach (var tank in _places)
            {
                if (tank != null)
                {
                    yield return tank;
                }
                else
                {
                    yield break;
                }
            }
        }
    }
}

